#ifndef NINE_FX_LIGHTING
#define NINE_FX_LIGHTING

struct NSurfaceDesc
{
	float4 diffuse;
	float4 ambient;
	float4 specular;
	float4 emissive;
	float power;
};

// wiato punktowe
struct NPointLight
{
	float3 position;	
	float4 color;
	float3 attenuation;
};

// wiato reflektorowe
struct NSpotLight
{
	float3 position;	
	float3 direction;
	float2 cosInnerOuterCone;
	float4 color;
	float3 attenuation;
};

// wiato kierunkowe
struct NDirLight
{
	float3 direction;
	float4 color;
};

/**************************************************/
/**************************************************/

// Funkcja oblicza zanik wiata
float CalculateAttenuation(float3 attenuation, float3 lightPosition, float3 position)
{
	float lightLength = distance(lightPosition, position);
	return 1.0f / (attenuation.x + attenuation.y * lightLength + attenuation.z * dot(lightLength, lightLength));
}
/**************************************************/

// Obliczanie efektu reflektorowego
float CalculateSpotCone(NSpotLight light, float3 dtLight)
{
	float cosLight = dot(-dtLight, light.direction);
	return smoothstep(light.cosInnerOuterCone.y, light.cosInnerOuterCone.x, cosLight);
}
/**************************************************/

// Funkcja oblicza diffuse i specular swiatla
void CalculateLighting(float3 position,			// Pozycja w przestrzeni oka
					   float3 normal,			// Normalna w przestrzeni oka
					   float specularPower,		// Moc rozbysku
					   float3 dtLight,			// Kierunek wiata
					   float4 lightColor,		// Kolor wiata						 
					   
					   out float4 oDiffuse,		// Wyjsciowy diffuse
					   out float4 oSpecular)	// Wyjsciowy specular
{
	// Staa z minimaln wartoci floata
	const float FLOAT_MIN_VALUE = 9.99999935e-038;

	// Podstawowe zmienne
	normal = normalize(normal);
	float3 dtCamera = normalize(-position);
	
	// Diffuse
	float diff = saturate(dot(normal, dtLight));
	oDiffuse = diff * lightColor;

	// Specular
	
	float3 specDir = normalize(dtLight + dtCamera);
	float spec = saturate(pow(max(FLOAT_MIN_VALUE, dot(specDir, normal)), specularPower));
	oSpecular = spec * lightColor;

	if(diff <= 0.0f)
	{
		oSpecular = 0.0f;
	}
}
/**************************************************/

// Funkcja oblicza kolor pochodzcy ze wiata punktowego
void CalculatePointLighting(float3 position,		// Pozycja w przestrzeni oka
							  float3 normal,		// Normalna w przestrzeni oka
							  float specularPower,	// Moc rozbysku
							  NPointLight light,		// Struktura wiata punktowego

							  out float4 oDiffuse,	// Wyjsciowy diffuse
							  out float4 oSpecular)	// Wyjsciowy specular
{
	float3 dtLight = normalize(light.position - position);

	CalculateLighting(position, normal, specularPower, dtLight, light.color, oDiffuse, oSpecular);

	oDiffuse *= CalculateAttenuation(light.attenuation, light.position, position);
	oSpecular  *= CalculateAttenuation(light.attenuation, light.position, position);
}
/**************************************************/

// Funkcja oblicza kolor pochodzcy ze wiata reflektorowego
void CalculateSpotLighting(float3 position,		// Pozycja w przestrzeni oka
						   float3 normal,		// Normalna w przestrzeni oka
						   float specularPower,	// Moc rozbysku
						   NSpotLight light,		// Struktura wiata reflektorowego

						   out float4 oDiffuse,	// Wyjsciowy diffuse
						   out float4 oSpecular)	// Wyjsciowy specular
{
	float3 dtLight = normalize(light.position - position);
	float scale = CalculateAttenuation(light.attenuation, light.position, position) * CalculateSpotCone(light, dtLight);

	CalculateLighting(position, normal, specularPower, dtLight, light.color, oDiffuse, oSpecular);
	
	oDiffuse *= scale;
	oSpecular  *= scale;
}
/**************************************************/

// Funkcja oblicza kolor pochodzcy ze wiata kierunkowego
void CalculateDirLighting(float3 position,		// Pozycja w przestrzeni oka
						  float3 normal,		// Normalna w przestrzeni oka
						  float specularPower,	// Moc rozbysku
						  NDirLight light,		// Struktura wiata kierunkowego

						  out float4 oDiffuse,	// Wyjsciowy diffuse
						  out float4 oSpecular)	// Wyjsciowy specular
{
	float3 dtLight = -light.direction;

	CalculateLighting(position, normal, specularPower, dtLight, light.color, oDiffuse, oSpecular);
}

#endif